# Generated by Django 1.10.6 on 2017-03-06 21:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('repository', '0002_auto_20161226_2320'),
    ]

    operations = [
        migrations.AlterField(
            model_name='elementsignature',
            name='method',
            field=models.CharField(choices=[('gpg', 'GnuPG'), ('x509', 'x509 (openSSL/libreSSL)')], db_index=True, max_length=10, verbose_name='méthode de signature'),
        ),
    ]
