# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-11-04 12:00
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('repository', '0004_auto_20171104_1257'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivestate',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
        migrations.AlterField(
            model_name='archivestate',
            name='creation',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='archivestate',
            name='modification',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='Modification date'),
        ),
        migrations.AlterField(
            model_name='archivestate',
            name='name',
            field=models.CharField(db_index=True, max_length=100, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='archivestate',
            name='repository',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='repository.Repository', verbose_name='Repository'),
        ),
        migrations.AlterField(
            model_name='archivestate',
            name='slug',
            field=models.SlugField(max_length=100, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='element',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
        migrations.AlterField(
            model_name='element',
            name='creation',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='element',
            name='extension',
            field=models.CharField(blank=True, db_index=True, default='', help_text='Automatically generated on creation', max_length=20, verbose_name='File extension'),
        ),
        migrations.AlterField(
            model_name='element',
            name='filename',
            field=models.CharField(blank=True, db_index=True, default='', help_text='Automatically generated on creation', max_length=255, verbose_name='filename'),
        ),
        migrations.AlterField(
            model_name='element',
            name='filesize',
            field=models.IntegerField(blank=True, db_index=True, default=0, help_text='Automatically generated on creation', verbose_name='File size'),
        ),
        migrations.AlterField(
            model_name='element',
            name='full_name',
            field=models.CharField(blank=True, db_index=True, max_length=255, verbose_name='Complete name'),
        ),
        migrations.AlterField(
            model_name='element',
            name='full_name_normalized',
            field=models.CharField(blank=True, db_index=True, help_text='Complete name without special chars nor accents', max_length=255, verbose_name='Normalized complete name'),
        ),
        migrations.AlterField(
            model_name='element',
            name='long_description',
            field=models.TextField(blank=True, verbose_name='Long description'),
        ),
        migrations.AlterField(
            model_name='element',
            name='md5',
            field=models.CharField(blank=True, db_index=True, default='', help_text='Automatically generated on creation', max_length=120, verbose_name='MD5 sum'),
        ),
        migrations.AlterField(
            model_name='element',
            name='modification',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='Modification date'),
        ),
        migrations.AlterField(
            model_name='element',
            name='name',
            field=models.CharField(db_index=True, max_length=100, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='element',
            name='official_link',
            field=models.URLField(blank=True, max_length=255, verbose_name='URL to the web page'),
        ),
        migrations.AlterField(
            model_name='element',
            name='repository',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='repository.Repository', verbose_name='Repository'),
        ),
        migrations.AlterField(
            model_name='element',
            name='sha1',
            field=models.CharField(blank=True, db_index=True, default='', help_text='Automatically generated on creation', max_length=120, verbose_name='SHA1 sum'),
        ),
        migrations.AlterField(
            model_name='element',
            name='sha256',
            field=models.CharField(blank=True, db_index=True, default='', help_text='Automatically generated on creation', max_length=120, verbose_name='SHA256 sum'),
        ),
        migrations.AlterField(
            model_name='element',
            name='short_description',
            field=models.CharField(blank=True, max_length=500, verbose_name='Short description'),
        ),
        migrations.AlterField(
            model_name='element',
            name='slug',
            field=models.SlugField(max_length=100, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='element',
            name='states',
            field=models.ManyToManyField(db_index=True, to='repository.ArchiveState', verbose_name='Archive states'),
        ),
        migrations.AlterField(
            model_name='elementsignature',
            name='creation',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='elementsignature',
            name='element',
            field=models.ForeignKey(default=0, on_delete=django.db.models.deletion.CASCADE, to='repository.Element', verbose_name='Element'),
        ),
        migrations.AlterField(
            model_name='elementsignature',
            name='method',
            field=models.CharField(choices=[('gpg', 'GnuPG'), ('x509', 'x509 (openSSL/libreSSL)')], db_index=True, max_length=10, verbose_name='signature method'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='admin_group',
            field=models.ManyToManyField(blank=True, db_index=True, related_name='repository_admin', to='auth.Group', verbose_name='Admin groups'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='archive_type',
            field=models.CharField(db_index=True, max_length=100, verbose_name='Repository type'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='creation',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='Creation date'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='is_private',
            field=models.BooleanField(db_index=True, default=False, verbose_name='Should readers be authenticated?'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='modification',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='Modification date'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='name',
            field=models.CharField(db_index=True, max_length=100, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='on_index',
            field=models.BooleanField(db_index=True, default=True, verbose_name='Display on public index?'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='reader_group',
            field=models.ManyToManyField(blank=True, db_index=True, related_name='repository_reader', to='auth.Group', verbose_name='Readers groups'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='slug',
            field=models.SlugField(max_length=100, verbose_name='Slug'),
        ),
    ]
