# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-11-04 13:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('repository', '0005_auto_20171104_1300'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='repository',
            options={'verbose_name': 'repository', 'verbose_name_plural': 'repositories'},
        ),
    ]
